@@ -51,7 +51,8 @@ __all__ = ['analysis',
 
 
 # The version number will eventually need to be set automatically from git
-release='0.9.8'
+release='0.9.8-1'
+__version__=release
 version='12131'
 
 import param 
