@@ -50,21 +50,15 @@ connections might have src_port = 'Recurrent', and probably also a
 special dest_port.
 
 
-$Id: simulation.py 11090 2010-06-22 11:09:01Z ceball $
+$Id$
 """
-__version__='$Revision: 11090 $'
 
 import param
-from param.parameterized import as_uninitialized,OptionalSingleton
 
 from copy import copy, deepcopy
-
+import time
 import bisect
 
-# JABALERT: Are these used for anything?
-SLEEP_EXCEPTION = "Sleep Exception"
-STOP = "Simulation Stopped"
-
 # CEBALERT: Is it dangerous to have 'Forever' implemented
 # like this? To start with, min(Forever,1) gives
 # FixedPoint('-1.00,2') i.e. Forever.
@@ -72,10 +66,10 @@ STOP = "Simulation Stopped"
 Forever = -1
 # (Python 2.6 includes support for float('inf') on all platforms?)
 
-# Default path to the current simulation, from main
-# Only to be used by script_repr(), to allow it to generate
-# a runnable script
-simulation_path="topo.sim"
+#: Default path to the current simulation, from main
+#: Only to be used by script_repr(), to allow it to generate
+#: a runnable script
+_simulation_path="topo.sim"
 
 
 
@@ -130,7 +124,7 @@ class EventProcessor(param.Parameterized):
 
         In the default implementation, a port is considered a match if
         the port is == to the key, but subclasses of EventProcessor can
-        override this to provide weaker forms of matching.
+        override this to allow weaker forms of matching.
         """
         return key in portlist
 
@@ -178,6 +172,27 @@ class EventProcessor(param.Parameterized):
         self.in_connections.append(conn)
 
 
+    def __dir__(self):
+        """
+        Extend dir() to include in_connections of an EventProcessor.
+        Useful for software that examines the list of possible
+        in_connections, such as tab completion in IPython.
+        """
+        default_dir = dir(type(self)) + list(self.__dict__)
+        return sorted(set(default_dir + [conn.name for conn in self.in_connections]))
+
+
+    def __getattr__(self, name):
+        """
+        Provide a simpler attribute-like syntax for accessing the
+        in_connections of an EventProcessor (e.g. obj.conn, for
+        in_connection "conn" of EventProcessor obj).
+        """
+        for conn in self.in_connections:
+            if conn.name == name: return conn
+        raise AttributeError
+
+
     def start(self):
         """        
         Called by the simulation when the EventProcessor is added to
@@ -204,7 +219,7 @@ class EventProcessor(param.Parameterized):
         data=deepcopy(data)
         for conn in out_conns_on_src_port:
             #self.verbose("Sending output on src_port %s via connection %s to %s" % (str(src_port), conn.name, conn.dest.name))
-            e=EPConnectionEvent(conn.delay+self.simulation.time(),conn,data,deep_copy=False)
+            e=EPConnectionEvent(self.simulation.convert_to_time_type(conn.delay)+self.simulation.time(),conn,data,deep_copy=False)
             self.simulation.enqueue_event(e)
             
 
@@ -229,7 +244,7 @@ class EventProcessor(param.Parameterized):
     
     def script_repr(self,imports=[],prefix="    "):
         """Generate a runnable command for creating this EventProcessor."""
-        return simulation_path+"['"+self.name+"']="+\
+        return _simulation_path+"['"+self.name+"']="+\
         super(EventProcessor,self).script_repr(imports=imports,prefix=prefix)
 
 
@@ -307,8 +322,11 @@ class EPConnection(param.Parameterized):
        that port.
        """)
 
-    delay = param.Number(default=0,doc=
-       """Simulation time between when each Event is generated by the src and when it is delivered to the dest.""")
+    # Should the lower bound be exclusive?
+    delay = param.Number(default=0.05,bounds=(0,None),doc="""
+       Simulation time between generation of an Event by the src and delivery to the dest.
+       Should normally be nonzero, to represent a causal with a well-defined ordering
+       of events.""")
 
     private = param.Boolean(default=False,doc=
        """Set to true if this connection is for internal use only, not to be manipulated by a user.""")
@@ -373,7 +391,7 @@ class EPConnection(param.Parameterized):
         mod = self.__module__
         imports.append("from %s import %s" % (mod,cls))
 
-        return simulation_path+".connect('"+self.src.name+"','"+self.dest.name+ \
+        return _simulation_path+".connect('"+self.src.name+"','"+self.dest.name+ \
                "',connection_type="+self.__class__.__name__+ \
                ",\n"+prefix+(",\n"+prefix).join(settings) + ")"
 
@@ -461,7 +479,7 @@ class CommandEvent(Event):
 
     def script_repr(self,imports=[],prefix="    "):
         """Generate a runnable command for creating this CommandEvent."""
-        return simulation_path+'.schedule_command('\
+        return _simulation_path+'.schedule_command('\
                +`self.time`+',"'+self.command_string+'")'
 
 
@@ -591,7 +609,6 @@ class PeriodicEventSequence(EventSequence):
 ### JP: Is it possible that some or all of this can be more cleanly
 ### implemented using PeriodicEvents?
 
-import time
 from math import floor
 class SomeTimer(param.Parameterized):
     """
@@ -689,6 +706,12 @@ class SomeTimer(param.Parameterized):
                                  name=self.func.__name__,
                                  duration=fduration,
                                  remaining=estimate)
+            
+            ## HACK refresh windows for camera in simulation time
+            import topo 
+            if hasattr(topo, 'guimain'):
+                topo.guimain.refresh_activity_windows()
+            ##
 
             if self.stop: break
 
@@ -725,6 +748,72 @@ class SomeTimer(param.Parameterized):
 
 
 
+# CEBALERT: This singleton-producing mechanism is pretty complicated,
+# and it would be great if someone could simplify it. Getting all of
+# the behavior we want for e.g. Simulation is tricky, but there are
+# tests for it. Note that:
+# (1) There should only ever be one single Simulation instance for
+#     which register is True. Creating, copying, and unpickling
+#     need to take this into account.
+# (2) A Simulation instance for which register is False should
+#     behave the same as any normal Python object.
+#
+# For how to use, see topo.base.simulation.Simulation or
+# topo.misc.commandline.GlobalParams.
+class OptionalSingleton(object):
+
+    _inst = None
+
+    def __new__(cls,singleton):
+        """
+        Return the single instance stored in _inst if singleton is
+        True; otherwise, return a new instance.
+        """
+        if singleton:
+            if cls is not type(cls._inst):
+                cls._inst = object.__new__(cls) 
+                cls._inst._singleton = True
+            return cls._inst
+        else:
+            new_inst = object.__new__(cls) 
+            new_inst._singleton = False
+            return new_inst
+
+    def __getnewargs__(self):
+        return (self._singleton,)
+
+    def __copy__(self):
+        # An OptionalSingleton(singleton=False) instance is copied, while the
+        # OptionalSingleton(singleton=True) instance is not copied.
+        if self._singleton:
+            return self
+        else:
+            # Ideally we'd just call "object.__copy__", but apparently
+            # there's no such method.
+            
+            # CB: I *think* this is how to do a copy. Any better
+            # ideas?  Python's copy.copy() function calls an object's
+            # __reduce__ method and then reconstructs the object from
+            # that using copy._reconstruct().
+            new_obj = self.__class__(self._singleton)
+            new_obj.__dict__ = copy(self.__dict__)
+            return new_obj 
+
+    def __deepcopy__(self,m):
+        if self._singleton:
+            return self
+        else:
+            new_obj = self.__class__(self._singleton)
+            new_obj.__dict__ = deepcopy(self.__dict__,m)
+            return new_obj
+
+    # CB: I might have bound __copy__ (& __deepcopy__) just to the
+    # Simulation(singleton=True) instance to avoid the Simulation
+    # class having a __copy__ method at all, but copy() only checks
+    # the *class* for the existence of __copy__.
+
+
+
 # Simulation stores its events in a linear-time priority queue (i.e., a
 # sorted list.) For efficiency, e.g. for spiking neuron simulations,
 # we'll probably need to replace the linear priority queue with a more
@@ -739,32 +828,53 @@ class Simulation(param.Parameterized,OptionalSingleton):
     Simulation is a singleton: there is only one instance of
     Simulation, no matter how many times it is instantiated.
     """
-    
+
+    # CEBALERT: Must use convert_to_time_type() everywhere a time is
+    # specified to ensure all times are the appropriate type. Relying
+    # on everyone to convert times to the time type seems dangerous!
+
     time_type = param.Parameter(default=float,constant=True,doc="""
-        The numeric type to use for Simulation's time values.
+        Callable for converting user-specified times into the numeric
+        type to be used for Simulation's time values.
         
-        Simulation's time type can be set to any numeric type that
-        supports the usual Python numeric operations, including at
-        least multiplication, addition, and subtraction.  Most of the
-        code assumes that fractional values are supported, as they are
-        for floats, but it may be possible to use an integer type
-        instead if various default fractional values are overridden.
+        Simulation's time can be set to any numeric type that supports
+        the usual Python numeric operations, including at least
+        multiplication, addition, and subtraction.  Most of the code
+        assumes that fractional values are supported, as they are for
+        floats, but it may be possible to use an integer type instead
+        if various default fractional values are overridden.
         
         For instance, one might wish to use arbitrary precision
-        floating-point time to avoid accumulating rounding errors.
-        For instance, if stepping through a simulation every 0.05 time
-        units, after 20 such steps using floats the simulation will
-        not reach 1.0 exactly, but will instead be slightly higher or
-        lower.  With an arbitrary precision float type such a series
-        of steps can be guaranteed to reach 1.0 exactly.
-        Alternatively, one might wish to use a rational type so that
-        events can be guaranteed to happen a certain number of times
-        in a given interval, even if the ratio cannot be expressed as
-        an even decimal or binary fraction.
-        
+        floating-point time to avoid accumulating rounding errors.  If
+        stepping through a simulation every 0.05 time units, after 20
+        such steps using floats the simulation will not reach 1.0
+        exactly, but will instead be slightly higher or lower.  With
+        an arbitrary precision float type such a series of steps can
+        be guaranteed to reach 1.0 exactly.  Alternatively, one might
+        wish to use a rational type so that events can be guaranteed
+        to happen a certain number of times in a given interval, even
+        if the ratio cannot be expressed as an even decimal or binary
+        fraction.
+                
+        time_type determines how the user can specify times. For
+        instance, if the user specifies a time as 0.05 (a float), this
+        will be represented in floating point as something like
+        0.050000000000000003. gmpy.mpq will correctly convert that to
+        mpq(1,20). Other types might require times to be specified as
+        strings (e.g. decimal.Decimal('0.05') in Python 2.6); in this
+        case the user would need to use a string whenever specifying a
+        time (e.g. a connection delay or a call to run() ). Some other
+        types will only produce the expected result if they are first
+        configured appropriately (e.g. decimal.Decimal(0.05) in Python
+        2.7 will give something like
+        Decimal('0.05000000000000000277555756156289135105907917022705078125')
+        by default, but this can be controlled by using an appropriate
+        decimal.Context).
+
         Extra arguments can be passed to time_type when it is being used
         to create the Simulation's time: see time_type_args.
-        
+
+
         Some potentially useful number classes::
         
         - gmpy.mpq: gmpy provides Python with access to the fast GNU
@@ -774,13 +884,18 @@ class Simulation(param.Parameterized,OptionalSingleton):
         - fixedpoint.FixedPoint: pure Python fixed-point number, but
           quite slow.
         
-        - Python's Decimal class: in the standard library, but also quite
-          slow.""")
+        - Python's decimal.Decimal and fractions.Fraction classes.
+        """)
 
+    # CEBALERT: probably should remove this and allow people to deal
+    # with functions requiring arguments
+    # (e.g. fixedpoint.FixedPoint(time,precision=4)) by currying or
+    # creating their own wrapper function. We wouldn't then need
+    # convert_to_time_type(), it would instead just be a call to
+    # time_type() (which should also be renamed).
     time_type_args = param.Parameter(default=(),constant=True,doc="""
-        Tuple of arguments to give to time_type whenever a new
-        instance of the time_type is being created (e.g. when a
-        Simulation is created).""")
+        Tuple of arguments passed to time_type after the first
+        argument (which is the time).""")
 
     register = param.Boolean(default=True,constant=True,doc="""
         Whether or not to register this Simulation. If True, this
@@ -818,7 +933,6 @@ class Simulation(param.Parameterized,OptionalSingleton):
     
     eps_to_start = []
 
-
     name = param.Parameter(constant=False)
 
 
@@ -910,26 +1024,18 @@ class Simulation(param.Parameterized,OptionalSingleton):
                 # name x...)
                 
                 
-
-    def _set_time(self,time=0):
+    # CEBALERT: if we're keeping this, should have a better name
+    def convert_to_time_type(self,time):
         """
-        Set this Simulation's _time to time, using the type specified
-        by the time_type parameter.
+        Convert the supplied time to the Simulation's time_type.
         """
         # pylint: disable-msg=W0201
         if self.time_type_args:
-            self._time = self.time_type(time,*self.time_type_args)
+            newtime = self.time_type(time,*self.time_type_args)
         else:
-            self._time = self.time_type(time)
+            newtime = self.time_type(time)
+        return newtime
 
-    @as_uninitialized
-    def set_time_type(self,time_type,*time_type_args):
-        """
-        Convenience method to allow time_type to be changed easily.
-        """
-        self.time_type = time_type
-        self.time_type_args = time_type_args
-        self._set_time(time=self._time)
 
     # Note that __init__ can still be called after the
     # Simulation(register=True) instance has been created. E.g. with
@@ -945,9 +1051,10 @@ class Simulation(param.Parameterized,OptionalSingleton):
         """
         Initialize a Simulation instance.
         """
+        self._time = self.convert_to_time_type(0)
+
         param.Parameterized.__init__(self,**params)
 
-        self._set_time()
 
         self._event_processors = {}
 
@@ -966,7 +1073,8 @@ class Simulation(param.Parameterized,OptionalSingleton):
         self.eps_to_start = []
         self.item_scale=1.0 # this variable determines the size of each item in a diagram
             
-        # CB: make this a parameter for documentation? Otherwise nobody will know
+        # CB (this comment applies to SomeTimer!): make this a
+        # parameter for documentation? Otherwise nobody will know
         # about being able to adjust step.
         #
         # we set step to 2 so that by default timing doesn't slow simulation too much. but
@@ -1064,9 +1172,34 @@ class Simulation(param.Parameterized,OptionalSingleton):
         for obj in self.objects(): yield obj
 
 
+    def __dir__(self):
+        """
+        Extend dir() to include simulation objects as well.  Useful
+        for software that examines the list of possible objects, such
+        as tab completion in IPython.
+        """
+        default_dir = dir(type(self)) + list(self.__dict__)
+        return sorted(set(default_dir + self.objects().keys()))
+
+
+    def __getattr__(self, name):
+        """
+        Provide a simpler attribute-like syntax for accessing objects
+        within a Simulation (e.g. sim.obj1, for an EventProcessor
+        "obj1" in sim).
+        """
+        if name=='_event_processors':
+            raise AttributeError
+
+        try:
+            return self.objects()[name]
+        except:
+            raise AttributeError
+
+
     def time(self):
         """
-        Return the current simulation time as a FixedPoint object.
+        Return the current simulation time.
         
         If the time returned will be used in the computation of a
         floating point variable, it should be cast into a floating
@@ -1082,6 +1215,8 @@ class Simulation(param.Parameterized,OptionalSingleton):
         users to control how much precision, etc. is used for time
         displays.
         """
+        # CEBALERT: I doubt this gets all attributes. Does it get
+        # properties (not that there are any right now)?
         all_vars = dict(self.get_param_values())
         all_vars.update(self.__dict__)
         if specified_time is not None:
@@ -1090,6 +1225,15 @@ class Simulation(param.Parameterized,OptionalSingleton):
         return timestr
 
 
+    @property
+    def timestr_prop(self):
+        """
+        A property that simply returns self.timestr(); useful for setting the
+        interactive command-line prompt.
+        """
+        return self.timestr()
+
+
     def basename(self):
         """
         Return a string suitable for labeling an object created
@@ -1147,8 +1291,8 @@ class Simulation(param.Parameterized,OptionalSingleton):
         # string to specify the time rather than a float (since float
         # is not compatible with all number types).
         
-        duration = self.time_type(duration,*self.time_type_args)
-        until = self.time_type(until,*self.time_type_args)
+        duration = self.convert_to_time_type(duration)
+        until = self.convert_to_time_type(until)
         
         
         # CEBHACKALERT: If I do topo.sim.run(10), then topo.sim.run(until=3),
@@ -1231,7 +1375,7 @@ class Simulation(param.Parameterized,OptionalSingleton):
         override this method as they wish, e.g. to wait for an
         external real time clock to advance first.
         """
-        self._time += delay
+        self._time += self.convert_to_time_type(delay)
 
     def enqueue_event(self,event):
         """
@@ -1262,7 +1406,7 @@ class Simulation(param.Parameterized,OptionalSingleton):
 
         The command should be a string.
         """
-        event = CommandEvent(time=time,command_string=command_string)
+        event = CommandEvent(time=self.convert_to_time_type(time),command_string=command_string)
         self.enqueue_event(event)
         
 
@@ -1400,12 +1544,10 @@ class Simulation(param.Parameterized,OptionalSingleton):
         objs  = [o.script_repr(imports=imports) for o in
                  sorted(self.objects().values(), cmp=lambda x, y: cmp(x.name,y.name))]
 
-        # CBENHANCEMENT: I have a function that compares conns based
-        # on name, then on src, then on dest, because lots of
-        # connections share the same name and I want the order to be
-        # the same every time for comparion. Can I just put that in
-        # here instead of cmp-by-name, or should we provide the chance
-        # for a user to plug in any sorting function?
+        # CBENHANCEMENT: could allow user to plug in a sorting
+        # function.  E.g. might want to compare conns based on name
+        # then dest then src if lots of conns share the same name (so
+        # the order is always the same).
         conns = [o.script_repr(imports=imports) for o in
                  sorted(self.connections(),      cmp=lambda x, y: cmp(x.name,y.name))]
 
@@ -1414,9 +1556,14 @@ class Simulation(param.Parameterized,OptionalSingleton):
                                cmp=lambda x, y: cmp(x.command_string,y.command_string)),
                         cmp=lambda x, y: cmp(x.time,y.time))]
 
+        # CEBALERT: hack to support importing the time type since the
+        # scheduled actions will have times printed using the
+        # time_type.
+        imports.append("from %s import %s"%(self.time_type.__module__,self.time_type.__name__))
+
         imps  = sorted(set(imports))
 
-        vals  = [simulation_path + "." + p + "=" + repr(getattr(self,p)) for p in
+        vals  = [_simulation_path + "." + p + "=" + repr(getattr(self,p)) for p in
                  ["name","startup_commands"]
                  if getattr(self,p)]
 
@@ -1456,6 +1603,78 @@ class Simulation(param.Parameterized,OptionalSingleton):
 
 
 
+class RealTimeSimulation(Simulation):
+    """
+    A (quasi) real-time simulation object.
+
+    This subclass of Simulation attempts to maintain a correspondence
+    between simulation time and real time, as defined by the timescale
+    parameter.  Real time simulation instances still maintain a
+    nominal, discrete simulation time that determines the order of
+    event delivery.
+
+    At the beginning of each simulation time epoch, the simulation
+    marks the actual wall clock time.  After event delivery for that
+    epoch has ended, the simulation calculates the amount of
+    computation time used for event processing, and executes a real
+    sleep for the remainder of the epoch.  If the computation time for
+    the epoch exceeded the real time, a warning is issued and
+    processing proceeds immediately to the next simulation time epoch.
+
+
+    RUN HOOKS
+
+    The simulation includes as parameters two lists of functions/callables,
+    run_start_hooks and run_stop_hooks, that will be called
+    immediately before and after event processing during a call to
+    .run().  This allows, for example, starting and stopping of
+    real-time devices that might use resources while the simulation is
+    not running.
+    """
+    
+    timescale = param.Number(default=1.0,bounds=(0,None),doc="""
+       The desired real length of one simulation time unit, in milliseconds.""")
+
+    run_start_hooks = param.HookList(default=[],doc="""
+       A list of callable objects to be called on entry to .run(),
+       before any events are processed.""")
+    
+    
+    run_stop_hooks = param.HookList(default=[],doc="""
+       A list of callable objects to be called on exit from .run()
+       after all events are processed.""") 
+
+    def __init__(self,**params):
+        super(RealTimeSimulation,self).__init__(**params)
+        self._real_timestamp = 0.0
+        
+    def run(self,*args,**kw):
+        for h in self.run_start_hooks:
+            h()
+        self._real_timestamp = self.real_time()
+        super(RealTimeSimulation,self).run(*args,**kw)
+        for h in self.run_stop_hooks:
+            h()
+
+    def real_time(self):
+        return time.time() * 1000
+    
+    def sleep(self,delay):
+        """
+        Sleep for the number of real milliseconds seconds corresponding to the
+        given delay, subtracting off the amount of time elapsed since the
+        last sleep.
+        """
+        sleep_ms = delay*self.timescale-(self.real_time()-self._real_timestamp)
+
+        if sleep_ms < 0:
+            self.warning("Realtime fault. Sleep delay of %f requires realtime sleep of %.2f ms."
+                         %(delay,sleep_ms))
+        else:
+            self.debug("sleeping. delay =",delay,"real delay =",sleep_ms,"ms.")
+            time.sleep(sleep_ms/1000.0)
+        self._real_timestamp = self.real_time()
+        self._time += delay
 
 
         
