@@ -20,41 +20,39 @@ set of parameters and some code, and only the bits that are specific
 to that particular plot or analysis appear below.  See the
 superclasses for the rest of the parameters and code.
 
-$Id: analysis.py 11154 2010-07-09 23:15:07Z jbednar $
+$Id$
 """
-__version__='$Revision: 11154 $'
 
-
-# JABALERT: Need to go through and eliminate unused imports.
-from math import pi, sin, cos
-from PIL import Image, ImageDraw
+import Image,ImageDraw
+    
 import copy
 
-from colorsys import hsv_to_rgb
 from numpy.oldnumeric import array, maximum
+from numpy import pi, sin, cos, nonzero, round, linspace
 
 import param
-from param.parameterized import PicklableClassAttributes, ParameterizedFunction
+from param.parameterized import ParameterizedFunction
 from param.parameterized import ParamOverrides
 
 import topo
-from topo.base.cf import CFSheet, CFProjection, Projection
-from topo.base.projection import ProjectionSheet
-from topo.base.simulation import EPConnectionEvent
+from topo.base.cf import Projection
 from topo.base.sheet import Sheet
-from topo.base.sheetview import SheetView
-from topo.command.basic import pattern_present, wipe_out_activity
-from topo.misc.util import frange
-from topo.misc.distribution import Distribution
-from topo.pattern.basic import Gaussian, GaussiansCorner, RawRectangle, Line
-from topo.pattern.random import GaussianRandom
 from topo.sheet import GeneratorSheet
-from topo.analysis.featureresponses import ReverseCorrelation, FeatureMaps
-from topo.plotting.plotgroup import create_plotgroup, plotgroups, ConnectionFieldsPlotGroup
+from topo.base.sheetview import SheetView
+from topo.misc.distribution import Distribution, DistributionStatisticFn
+from topo.misc.distribution import DSF_MaxValue, DSF_BimodalPeaks
+from topo.misc.distribution import DSF_WeightedAverage, DSF_VonMisesFit, DSF_BimodalVonMisesFit
+from topo.pattern import GaussiansCorner, Gaussian, RawRectangle, Composite, Constant
+from topo.analysis.featureresponses import ReverseCorrelation
+from topo.plotting.plotgroup import create_plotgroup, plotgroups
 
-from topo.plotting.plotgroup import UnitMeasurementCommand,ProjectionSheetMeasurementCommand,default_input_sheet
-from topo.analysis.featureresponses import Feature, PatternPresenter, Subplotting
+from topo.plotting.plotgroup import UnitMeasurementCommand,ProjectionSheetMeasurementCommand
+from topo.analysis.featureresponses import Feature, PatternPresenter, MeasureResponseCommand
 from topo.analysis.featureresponses import SinusoidalMeasureResponseCommand, PositionMeasurementCommand, SingleInputResponseCommand
+from topo.base.patterngenerator import PatternGenerator
+from topo.command import pattern_present
+
+from topo.misc.patternfn import line
 
 
 # Helper function for save_plotgroup
@@ -159,7 +157,7 @@ class save_plotgroup(ParameterizedFunction):
 
 
 
-def decode_feature(sheet, preference_map = "OrientationPreference", axis_bounds=(0.0,1.0), cyclic=True, weighted_average=True):
+def decode_feature(sheet, preference_map = "OrientationPreference", axis_bounds=(0.0,1.0), cyclic=True, weighted_average=True, cropfn=lambda(x):x):
     """
     Estimate the value of a feature from the current activity pattern on a sheet.
     
@@ -180,20 +178,38 @@ def decode_feature(sheet, preference_map = "OrientationPreference", axis_bounds=
     quantity), then the result will be the vector average of the
     activated orientations.  For an orientation map this value should
     be an estimate of the orientation present on the input.
+
+    If desired, a cropfn can be supplied that will narrow the analysis
+    down to a specific region of the array; this function will be
+    applied to the preference_map and to the activity array before
+    decoding.  Examples:
+
+    Decode whole area:
+ 
+       decode_feature(topo.sim["V1"])
+    
+    Decode left half only:
+
+       r,c = topo.sim["V1"].activity.shape
+       lefthalf  = lambda(x): x[:,0:c/2]
+       righthalf = lambda(x): x[:,c/2:]
+
+       decode_feature(topo.sim["V1"], cropfn=lefthalf)
+    
     """
+
     d = Distribution(axis_bounds, cyclic)
     
     if not (preference_map in sheet.sheet_views):
-        topo.sim.warning(preference_map + " should be measured before calling decode_orientations.")
+        topo.sim.warning(preference_map + " should be measured before calling decode_feature.")
     else:
-        map = sheet.sheet_views[preference_map]
-        d.add(dict(zip(map.view()[0].ravel(), sheet.activity.ravel())))
+        v = sheet.sheet_views[preference_map]
+        d.add(dict(zip(cropfn(v.view()[0]).ravel(), 
+                       cropfn(sheet.activity).ravel())))
+    
+    res = DSF_WeightedAverage()(d) if weighted_average else DSF_MaxValue()(d)
+    return res['']['preference']
     
-    if weighted_average:
-        return d.weighted_average()
-    else:
-        return d.max_value_bin()
-
 
 
 def update_activity():
@@ -326,8 +342,20 @@ class measure_rfs(SingleInputResponseCommand):
     some of the time there is a pattern on the input.
     """  
     static_parameters = param.List(default=["offset","size"])
-    __abstract = True
+    
+    sampling = param.Number(default=1.0,bounds=(1.0,None),doc="""
+    	The sampling value determines the size of the input pattern and
+    	the number of presentations required to fully sample the input sheet.
+    	The higher the value the coarser the receptive field measurement 
+    	will be.""")
+    
+    area_ratio = param.Number(default=1.0,bounds=(0.0,1.0),doc="""
+    	Ratio defining the area in the input sheet that is sampled during
+    	the reverse correlation procedure. Reducing this value below 1.0
+    	will invalidate the RFs of all neurons outside the specified area.""")
 
+    __abstract = True
+    
     def __call__(self,**params):
         p=ParamOverrides(self,params)
         self.params('input_sheet').compute_default()
@@ -342,16 +370,15 @@ class measure_rfs(SingleInputResponseCommand):
           
     def _feature_list(self,p):
     
-        # Obtain sheet dimensions and density.
         left, bottom, right, top = p.input_sheet.nominal_bounds.lbrt()
+        left *= p.area_ratio; bottom *= p.area_ratio; right *= p.area_ratio; top *= p.area_ratio
         sheet_density = float(p.input_sheet.nominal_density)
         
-        # Cannot assume square sheet so two independent values for axes divisions.
-        vertical_divisions = (sheet_density * (top - bottom)) - 1
-        horizontal_divisions = (sheet_density * (right - left)) - 1
+        # Cannot assume square sheet.
+        vertical_divisions = ((sheet_density * (top - bottom)) - 1) / p.sampling
+        horizontal_divisions = ((sheet_density * (right - left)) - 1) / p.sampling
 
-        # Calculate size of a division.
-        unit_size = 1.0 / sheet_density
+        unit_size = 1.0 / sheet_density * p.sampling
         half_unit_size = unit_size / 2.0 # saves repeated calculation.
         p['size'] = unit_size
         
@@ -435,13 +462,16 @@ class measure_sine_pref(SinusoidalMeasureResponseCommand):
         # Always varies frequency and phase; everything else depends on parameters.
 
         features = \
-            [Feature(name="frequency",values=p.frequencies)]
+            [Feature(name="frequency",values=p.frequencies,
+                    preference_fn=DSF_WeightedAverage())]
 
         if p.num_direction==0: features += \
-            [Feature(name="orientation",range=(0.0,pi),step=pi/p.num_orientation,cyclic=True)]
+            [Feature(name="orientation",range=(0.0,pi),step=pi/p.num_orientation,
+                            cyclic=True, preference_fn=self.preference_fn)]
 
         features += \
-            [Feature(name="phase",range=(0.0,2*pi),step=2*pi/p.num_phase,cyclic=True)]
+            [Feature(name="phase",range=(0.0,2*pi),step=2*pi/p.num_phase,cyclic=True,
+                     preference_fn=DSF_WeightedAverage( value_scale=(0., 1/2.0/pi) ))]
 
         if p.num_ocularity>1: features += \
             [Feature(name="ocular",range=(0.0,1.0),step=1.0/p.num_ocularity)]
@@ -464,7 +494,7 @@ class measure_sine_pref(SinusoidalMeasureResponseCommand):
             or_values = list(set([compute_orientation_from_direction([("direction",v)]) for v in dr.values]))
             features += [dr, \
                  Feature(name="orientation",range=(0.0,pi),values=or_values,cyclic=True,
-                         compute_fn=compute_orientation_from_direction)]
+                         compute_fn=compute_orientation_from_direction,preference_fn=self.preference_fn)]
 
         return features
 
@@ -474,16 +504,25 @@ class measure_or_pref(SinusoidalMeasureResponseCommand):
     """Measure an orientation preference map by collating the response to patterns."""
 
     subplot = param.String("Orientation")
+
+    preference_fn = param.ClassSelector( DistributionStatisticFn, 
+        default=DSF_WeightedAverage(value_scale=(0.0,1.0/pi)), doc=""" 
+        Function that will be used to analyze the distributions of unit
+        responses. Sets value_scale to normalize orientation preference
+        values.""" )
     
     def _feature_list(self,p):
+
         return [Feature(name="frequency",values=p.frequencies),
-                Feature(name="orientation",range=(0.0,pi),step=pi/p.num_orientation,cyclic=True),
+                Feature(name="orientation",range=(0.0,pi),step=pi/p.num_orientation,
+                        preference_fn=self.preference_fn,cyclic=True),
                 Feature(name="phase",range=(0.0,2*pi),step=2*pi/p.num_phase,cyclic=True)]
 
 
 pg= create_plotgroup(name='Orientation Preference',category="Preference Maps",
              doc='Measure preference for sine grating orientation.',
-             pre_plot_hooks=[measure_sine_pref.instance()])
+             pre_plot_hooks=[measure_sine_pref.instance(
+                 preference_fn=DSF_WeightedAverage( value_scale=(0., 1./pi) ))] )
 pg.add_plot('Orientation Preference',[('Hue','OrientationPreference')])
 pg.add_plot('Orientation Preference&Selectivity',
             [('Hue','OrientationPreference'), ('Confidence','OrientationSelectivity')])
@@ -493,9 +532,53 @@ pg.add_plot('Phase Selectivity',[('Strength','PhaseSelectivity')])
 pg.add_static_image('Color Key','command/or_key_white_vert_small.png')
 
 
+pg= create_plotgroup(name='vonMises Orientation Preference',category="Preference Maps",
+             doc='Measure preference for sine grating orientation using von Mises fit.',
+             pre_plot_hooks=[measure_sine_pref.instance(
+                 preference_fn=DSF_VonMisesFit( value_scale=(0., 1./pi) ),
+                 num_orientation=16)])
+pg.add_plot('Orientation Preference',[('Hue','OrientationPreference')])
+pg.add_plot('Orientation Preference&Selectivity',
+            [('Hue','OrientationPreference'), ('Confidence','OrientationSelectivity')])
+pg.add_plot('Orientation Selectivity',[('Strength','OrientationSelectivity')])
+pg.add_plot('Phase Preference',[('Hue','PhasePreference')])
+pg.add_plot('Phase Selectivity',[('Strength','PhaseSelectivity')])
+pg.add_static_image('Color Key','command/or_key_white_vert_small.png')
+
+
+pg= create_plotgroup(name='Bimodal Orientation Preference', category="Preference Maps",
+             doc='Measure preference for sine grating orientation using bimodal von Mises fit.',
+             pre_plot_hooks=[measure_sine_pref.instance(
+                 preference_fn=DSF_BimodalVonMisesFit( value_scale=(0., 1./pi) ),
+                 num_orientation=16)])
+pg.add_plot('Orientation Preference',[('Hue','OrientationPreference')])
+pg.add_plot('Orientation Preference&Selectivity',
+            [('Hue','OrientationPreference'), ('Confidence','OrientationSelectivity')])
+pg.add_plot('Orientation Selectivity',[('Strength','OrientationSelectivity')])
+pg.add_plot('Second Orientation Preference', [('Hue','OrientationMode2Preference')])
+pg.add_plot('Second Orientation Preference&Selectivity',
+            [('Hue','OrientationMode2Preference'), ('Confidence','OrientationMode2Selectivity')])
+pg.add_plot('Second Orientation Selectivity', [('Strength','OrientationMode2Selectivity')])
+pg.add_static_image('Color Key', 'command/or_key_white_vert_small.png')
+
+        
+pg = create_plotgroup(name='Two Orientation Preferences',category='Preference Maps',
+    doc='Display the two most preferred orientations for each units, using bimodal von Mises fit.',
+    pre_plot_hooks=[measure_sine_pref.instance(
+         preference_fn=DSF_BimodalVonMisesFit(), num_orientation=16)])
+pg.add_plot( 'Two Orientation Preferences', [
+		( 'Or1',	'OrientationPreference' ),
+		( 'Sel1',	'OrientationSelectivity' ),
+		( 'Or2',	'OrientationMode2Preference' ),
+		( 'Sel2',	'OrientationMode2Selectivity' )
+])
+pg.add_static_image('Color Key','command/two_or_key_vert.png')
+
+
 pg= create_plotgroup(name='Spatial Frequency Preference',category="Preference Maps",
              doc='Measure preference for sine grating orientation and frequency.',
-             pre_plot_hooks=[measure_sine_pref.instance()])
+             pre_plot_hooks=[measure_sine_pref.instance(
+                 preference_fn=DSF_WeightedAverage( value_scale=(0., 1./pi) ))] )
 pg.add_plot('Spatial Frequency Preference',[('Strength','FrequencyPreference')])
 pg.add_plot('Spatial Frequency Selectivity',[('Strength','FrequencySelectivity')])
 # Just calls measure_sine_pref to plot different maps.
@@ -621,13 +704,72 @@ pg.add_plot('Hue Selectivity',[('Strength','HueSelectivity')])
 
 
 
-
-gaussian_corner = topo.pattern.basic.Composite(
+gaussian_corner = Composite(
     operator = maximum, generators = [
-        topo.pattern.basic.Gaussian(size = 0.06,orientation=0,aspect_ratio=7,x=0.3),
-        topo.pattern.basic.Gaussian(size = 0.06,orientation=pi/2,aspect_ratio=7,y=0.3)])
+        Gaussian(size = 0.06,orientation=0,aspect_ratio=7,x=0.3),
+        Gaussian(size = 0.06,orientation=pi/2,aspect_ratio=7,y=0.3)])
 
 
+class measure_second_or_pref(SinusoidalMeasureResponseCommand):
+    """Measure the secondary  orientation preference maps."""
+
+    num_orientation	= param.Integer( default=16, bounds=(1,None), softbounds=(1,64),
+                                    doc="Number of orientations to test.")
+    true_peak 	 	= param.Boolean( default=True, doc="""If set the second
+	    orientation response is computed on the true second mode of the
+	    orientation distribution, otherwise is just the second maximum response""" ) 
+
+    subplot		= param.String("Second Orientation")
+    
+    def _feature_list(self, p):
+    	fs	= [ Feature(name="frequency", values=p.frequencies) ]
+    	if p.true_peak:
+	    fs.append(
+		Feature(name="orientation", range=(0.0, pi), step=pi/p.num_orientation,
+                    cyclic=True, preference_fn=DSF_BimodalPeaks() ) )
+	else:
+	    fs.append(
+		Feature(name="orientation", range=(0.0, pi), step=pi/p.num_orientation,
+                    cyclic=True, preference_fn=DSF_BimodalPeaks() ) )
+	fs.append( Feature(name="phase", range=(0.0, 2*pi), step=2*pi/p.num_phase, cyclic=True) )
+
+	return fs
+
+
+pg= create_plotgroup(name='Second Orientation Preference', category="Preference Maps",
+             doc='Measure the second preference for sine grating orientation.',
+             pre_plot_hooks=[measure_second_or_pref.instance( true_peak=False )])
+pg.add_plot('Second Orientation Preference', [('Hue','OrientationMode2Preference')])
+pg.add_plot('Second Orientation Preference&Selectivity',
+            [('Hue','OrientationMode2Preference'), ('Confidence','OrientationMode2Selectivity')])
+pg.add_plot('Second Orientation Selectivity', [('Strength','OrientationMode2Selectivity')])
+pg.add_static_image('Color Key', 'command/or_key_white_vert_small.png')
+
+
+pg= create_plotgroup(name='Second Peak Orientation Preference',category="Preference Maps",
+             doc='Measure the second peak preference for sine grating orientation.',
+             pre_plot_hooks=[measure_second_or_pref.instance( true_peak=True )])
+pg.add_plot('Second Peak Orientation Preference', [('Hue','OrientationMode2Preference')])
+pg.add_plot('Second Peak Orientation Preference&Selectivity',
+            [('Hue','OrientationMode2Preference'), ('Confidence','OrientationMode2Selectivity')])
+pg.add_plot('Second Peak Orientation Selectivity', [('Strength','OrientationMode2Selectivity')])
+pg.add_static_image('Color Key','command/or_key_white_vert_small.png')
+
+        
+pg = create_plotgroup(name='Two Peaks Orientation Preferences',category='Preference Maps',
+    doc="""Display the two most preferred orientations for all units with a
+    multimodal orientation preference distribution.""",
+    pre_plot_hooks=[
+    		measure_second_or_pref.instance(num_orientation=16, true_peak=True)
+])
+pg.add_plot( 'Two Peaks Orientation Preferences', [
+		( 'Or1',	'OrientationPreference' ),
+		( 'Sel1',	'OrientationSelectivity' ),
+		( 'Or2',	'OrientationMode2Preference' ),
+		( 'Sel2',	'OrientationMode2Selectivity' )
+])
+pg.add_static_image('Color Key','command/two_or_key_vert.png')
+
 class measure_corner_or_pref(PositionMeasurementCommand):
     """Measure a corner preference map by collating the response to patterns."""
     
@@ -650,14 +792,18 @@ class measure_corner_or_pref(PositionMeasurementCommand):
     def _feature_list(self,p):
         width =1.0*p.x_range[1]-p.x_range[0]
         height=1.0*p.y_range[1]-p.y_range[0]
-        return [Feature(name="x",range=p.x_range,step=width/p.divisions),
-                Feature(name="y",range=p.y_range,step=height/p.divisions),
-                Feature(name="orientation",range=(0,2*pi),step=2*pi/p.num_orientation,cyclic=True)]
+        return [
+            Feature(name="x",range=p.x_range,step=width/p.divisions,preference_fn=self.preference_fn),
+            Feature(name="y",range=p.y_range,step=height/p.divisions,preference_fn=self.preference_fn),
+            Feature(name="orientation",range=(0,2*pi),step=2*pi/p.num_orientation,cyclic=True,
+                preference_fn=DSF_WeightedAverage(value_scale=(0., 1/2.0/pi))
+            )]
 
 
 pg= create_plotgroup(name='Corner OR Preference',category="Preference Maps",
              doc='Measure orientation preference for corner shape (or other complex stimuli that cannot be represented as fullfield patterns).',
-             pre_plot_hooks=[measure_corner_or_pref.instance()],
+             pre_plot_hooks=[measure_corner_or_pref.instance(
+                 preference_fn=DSF_WeightedAverage())],
              normalize='Individually')
 pg.add_plot('Corner Orientation Preference',[('Hue','OrientationPreference')])
 pg.add_plot('Corner Orientation Preference&Selectivity',[('Hue','OrientationPreference'),
@@ -714,12 +860,11 @@ class measure_corner_angle_pref(PositionMeasurementCommand):
 	a_step	= ( angle_1 - angle_0 ) / float( p.num_angle - 1 )
 	self._make_key_image( p )
         return [
-            Feature( name = "x",           range = p.x_range, step = x_step ),
-            Feature( name = "y",           range = p.y_range, step = y_step ),
-            Feature( name = "orientation", range = (0, 2*pi), step = o_step, cyclic = True ),
-            Feature( name = "angle",       range = a_range,   step = a_step,
-                     value_offset     = - angle_0,
-                     value_multiplier = 1. / ( angle_1 - angle_0 ) )
+            Feature( name="x",           range=p.x_range, step=x_step ),
+            Feature( name="y",           range=p.y_range, step=y_step ),
+            Feature( name="orientation", range=(0, 2*pi), step=o_step, cyclic=True ),
+            Feature( name="angle",       range=a_range,   step=a_step,
+                    preference_fn=DSF_WeightedAverage( value_scale=( -angle_0, 1./(angle_1-angle_0) ) ) )
 	]
 
 
@@ -744,7 +889,7 @@ class measure_corner_angle_pref(PositionMeasurementCommand):
 	y_0	= border + y_d
 	l	= 15
 
-	hues	= [ "hsl(%2d,100%%,50%%)" % h		for h in range( 0, 255, 255 / n_a) ]
+	hues	= [ "hsl(%2d,100%%,50%%)" % h		for h in range( 0, 360, 360 / n_a ) ]
 	angles	= [ 0.5*angle_0 + a_step * a		for a in range( n_a ) ]
 	y_pos	= [ int( round( y_0 + y * y_step ) )	for y in range( n_a ) ]
 	deltas	= [ ( int( round( l * cos( a ) ) ), int( round( l * sin( a ) ) ) )
@@ -777,30 +922,343 @@ pg.add_plot('Corner Orientation Selectivity',[('Strength','OrientationSelectivit
 pg.add_static_image( 'Hue Code', measure_corner_angle_pref.instance().key_img_fname )
 
 
-# Measure sound frequency preference maps
-class measure_frequency_pref(PositionMeasurementCommand):
-    """Measure a frequency preference and selectivity map"""
+
+class PatternPresenter2(param.Parameterized):
+
+    apply_output_fns = param.Boolean(default = True, 
+        doc = """When presenting a pattern, whether to apply each 
+        sheet's output function.  If False, for many networks the 
+        response will be linear, which requires fewer test patterns to 
+        measure a meaningful response, but it may not correspond to the 
+        actual preferences of each neuron under other conditions. If 
+        True, callers will need to ensure that the input patterns are 
+        in a suitable range to drive the neurons to generate meaningful
+        output, because e.g. a threshold-based output function might
+        result in no activity for inputs that are too weak..""")
+
+    duration = param.Number(default = 1.0,
+        doc = """Amount of simulation time for which to present each 
+        test pattern. By convention, most Topographica example files 
+        are 
+        designed to have a suitable activity pattern computed by the
+        default time, but the duration will need to be changed for
+        other models that do not follow that convention or if a
+        linear response is desired.""")
+
+    # CEBALERT: generator_sheets=[] is probably a surprising way of
+    # actually getting all the generator sheets.
+    generator_sheets = param.List(default = [], 
+        doc="""The set of GeneratorSheets onto which patterns will be 
+        drawn. By default (i.e. for an empty list), all GeneratorSheets 
+        in the simulation will be used.""")
+        
+    pattern_generator = param.Parameter(default=Constant(),
+        doc = """The PatternGenerator that will be drawn on the generator 
+        sheets (the parameters of the pattern_generator are specified 
+        during calls).""")
+
+
+    def __call__(self, features_values, param_dict):
+        
+        for param, value in param_dict.iteritems():
+            setattr(self.pattern_generator, param, value)                               
+                              
+        for feature, value in features_values.iteritems():
+            setattr(self.pattern_generator, feature, value)                               
+        
+        all_input_sheet_names = topo.sim.objects(GeneratorSheet).keys()
+
+        if len(self.generator_sheets) > 0:
+            input_sheet_names = [sheet.name for sheet in self.generator_sheets]
+        else:
+            input_sheet_names = all_input_sheet_names
+        
+        # Copy the given generator once for every GeneratorSheet
+        inputs = dict.fromkeys(input_sheet_names)
+        for key in inputs.keys():
+            inputs[key] = copy.deepcopy(self.pattern_generator)
+            
+        self._custom_presenter(inputs, input_sheet_names)
+        
+        # blank patterns for unused generator sheets
+        for sheet_name in set(all_input_sheet_names).difference(set(input_sheet_names)):
+            inputs[sheet_name] = Constant(scale=0)
+                
+        pattern_present(inputs, self.duration, plastic=False, apply_output_fns=self.apply_output_fns)                
+                        
+
+    def _custom_presenter(self, inputs, input_sheet_names):
+        """This method provides a minimum overload for performing custom actions
+        with a pattern presenter."""
+
+
+
+class frequency_mapper(PatternGenerator):
+    """Activates a generator sheet at the specified frequency (for all latencies).
+    It does so by translating from a frequency to a y position and presenting a line
+    at that y position (of the specified size)."""
+    
+    __abstract = True
+            
+    size = param.Number(default=0.01, bounds=(0.0,None), softbounds=(0.0,1.0), 
+        doc="Thickness (width) of the frequency band for every presentation.")
+        
+    frequency_spacing = param.Array(default=None,
+        doc="""The spacing of the available frequency range, this allows us to define
+        (and hence map) a non linear spacing by specifying the frequency value at each
+        sheet unit.""")
+    
+    
+    def __init__(self, **params):
+        super(frequency_mapper, self).__init__(**params) 
+        self.frequency_spacing = round(self.frequency_spacing)
+
+            
+    def getFrequency(self):
+        sheet_range = self.bounds.lbrt()
+        y_range = sheet_range[3] - sheet_range[1]
+        
+        index = ((self.y - sheet_range[1]) / y_range) * (len(self.frequency_spacing) - 1)
+
+        return self.frequency_spacing[index]
+        
+        
+    def setFrequency(self, new_frequency):
+        index = nonzero(self.frequency_spacing >= new_frequency)[0][0]
+        
+        sheet_range = self.bounds.lbrt()
+        y_range = sheet_range[3] - sheet_range[1]
+        
+        ratio = (len(self.frequency_spacing) - 1) / y_range
+        
+        y = (index / ratio) + sheet_range[1]
+        setattr(self, 'y', y)
+
+
+    frequency = property(getFrequency, setFrequency, "The frequency at which to present.")
+    
+    
+    def function(self, p):
+        return line(self.pattern_y, p.size, 0.001)
+
+
+
+class measure_frequency_preference(MeasureResponseCommand):
+    """Measure a best frequency preference and selectivity map for auditory neurons."""
         
     display = param.Boolean(True) 
-    pattern_presenter = param.Callable(PatternPresenter(Line(smoothing=0.0001,thickness=0.05)))
+    static_parameters = param.List(default=["scale", "offset"])        
     
-    # BK-ALERT: These are hard coded to the lissom audio sheet dimensions.
-    # i'm not sure how to avoid that, PositionMeasurementCommand isn't
-    # actually able to access the sheet dimensions.
-    y_range = param.NumericTuple((-0.5,0.5))
-    divisions = param.Integer(100)
     
     def _feature_list(self,p):
-        return [Feature(name="x", values=[0.0]), 
-                Feature(name="y", range=p.y_range, step=(p.y_range[1]-p.y_range[0])/float(p.divisions))]
+        input_sheets = topo.sim.objects(GeneratorSheet).values()
+        
+        # BK-NOTE: if anyone wants to generalise this method for heterogeneous sheets, by all means do so,
+        # for my personal use the additional code required to generalise was overkill.
+        for sheet in range(1, len(input_sheets)-1):
+            assert input_sheets[sheet].bounds == input_sheets[sheet-1].bounds
+            assert input_sheets[sheet].xdensity == input_sheets[sheet-1].xdensity
+            assert input_sheets[sheet].ydensity == input_sheets[sheet-1].ydensity
+        
+        divisions = float(input_sheets[0].ydensity)
+
+        try:
+            min_frequency = input_sheets[0].input_generator.min_frequency
+            max_frequency = input_sheets[0].input_generator.max_frequency
+            frequency_spacing = input_sheets[0].input_generator.frequency_spacing
+        except AttributeError:
+            min_frequency = 1
+            max_frequency = int(divisions) + 1
+            frequency_spacing = linspace(min_frequency, max_frequency, num=divisions+1, endpoint=True)
+            self.warning("Input generator is missing min_frequency, max_frequency, or frequency_spacing - will present linearly from", str(min_frequency), "to", str(max_frequency), "instead.")
+
+        generator = frequency_mapper(size=1.0/divisions, frequency_spacing=frequency_spacing)
+        self.pattern_presenter = PatternPresenter2(pattern_generator=generator)
+        
+        return [Feature(name="frequency", range=(min_frequency,max_frequency), step=1)]
 
 
-pg= create_plotgroup(name='Frequency Preference and Selectivity',category="Preference Maps",
-                     pre_plot_hooks=[measure_frequency_pref.instance()], normalize='Individually',
-                     doc='Measure best frequency preference and selectivity for auditory neurons.')
+pg= create_plotgroup(name='Frequency Preference and Selectivity', category="Auditory",
+    pre_plot_hooks=[measure_frequency_preference.instance()], normalize='Individually',
+    doc='Measure a best frequency preference and selectivity map for auditory neurons.')
+
+pg.add_plot('[Frequency Preference]', [('Strength','FrequencyPreference')])
+pg.add_plot('[Frequency Selectivity]', [('Strength','FrequencySelectivity')])
+
+
+
+class log_frequency_mapper(PatternGenerator):
+    """Activates a generator sheet at the specified frequency (for all latencies).
+        It does so by translating from a frequency to a y position and presenting a line
+        at that y position (of the specified size)."""
+
+    __abstract = True
+    
+    size = param.Number(default=0.01, bounds=(0.0,None), softbounds=(0.0,1.0), 
+        doc="Thickness (width) of the frequency band for every presentation.")
+    
+    frequency_spacing = param.Array(default=None,
+        doc="""The spacing of the available frequency range, this allows us to define
+            (and hence map) a non linear spacing by specifying the frequency value at each
+            sheet unit.""")
+    
+    
+    def __init__(self, **params):
+        super(log_frequency_mapper, self).__init__(**params) 
+        self.frequency_spacing = round(self.frequency_spacing)
+    
+    
+    def getFrequency(self):
+        sheet_min = (self.bounds.lbrt())[1]
+        index = (self.y - sheet_min) * (self.frequency_spacing.size - 1.0)
+        return self.frequency_spacing[index]
+    
+    
+    def setFrequencyBand(self, new_frequency_band):
+        sheet_min = (self.bounds.lbrt())[1]
+        y = sheet_min + (new_frequency_band / (self.frequency_spacing.size - 1.0))
+        setattr(self, 'y', y)
+    
+    
+    frequency = property(getFrequency, setFrequencyBand, "The log frequency band at which to present.")
+    
+    
+    def function(self, p):
+        return line(self.pattern_y, p.size, 0.001)
+        
+        
+        
+class measure_log_frequency_preference(MeasureResponseCommand):
+    """Measure a best frequency preference and selectivity map for auditory neurons."""
+    
+    display = param.Boolean(True) 
+    static_parameters = param.List(default=["scale", "offset"])        
+    
+    
+    def _feature_list(self,p):
+        input_sheets = topo.sim.objects(GeneratorSheet).values()
+        
+        # BK-NOTE: if anyone wants to generalise this method for heterogeneous sheets, by all means do so,
+        # for my personal use the additional code required to generalise was overkill.
+        for sheet in range(1, len(input_sheets)-1):
+            assert input_sheets[sheet].bounds == input_sheets[sheet-1].bounds
+            assert input_sheets[sheet].xdensity == input_sheets[sheet-1].xdensity
+            assert input_sheets[sheet].ydensity == input_sheets[sheet-1].ydensity
+        
+        divisions = float(input_sheets[0].ydensity)
+        
+        try:
+            min_frequency = input_sheets[0].input_generator.min_frequency
+            max_frequency = input_sheets[0].input_generator.max_frequency
+            frequency_spacing = input_sheets[0].input_generator.frequency_spacing
+        except AttributeError:
+            min_frequency = 1
+            max_frequency = int(divisions) + 1
+            frequency_spacing = linspace(min_frequency, max_frequency, num=divisions+1, endpoint=True)
+            self.warning("Input generator is missing min_frequency, max_frequency, or frequency_spacing - will present linearly from", str(min_frequency), "to", str(max_frequency), "instead.")
+        
+        generator = log_frequency_mapper(size=1.0/divisions, frequency_spacing=frequency_spacing)
+        self.pattern_presenter = PatternPresenter2(pattern_generator=generator)        
+        
+        return [Feature(name="frequency", range=(0,divisions), step=1)]
+
+
+pg= create_plotgroup(name='Log Frequency Band Preference and Selectivity', category="Auditory",
+                     pre_plot_hooks=[measure_log_frequency_preference.instance()], normalize='Individually',
+                     doc='Measure a best frequency preference and selectivity map for auditory neurons (presents frequencies with logarithmically increasing bandwidth).')
+
+pg.add_plot('[Log Frequency Band Preference]', [('Strength','FrequencyPreference')])
+pg.add_plot('[Log Frequency Band Selectivity]', [('Strength','FrequencySelectivity')])
+
+
+
+class latency_mapper(PatternGenerator):
+    """Activates a generator sheet at the specified latency (for all frequencies).
+    It does so by translating from a latency to an x position and presenting a line
+    at that x position (of the specified size)."""
+    
+    __abstract = True
+
+    size = param.Number(default=0.01, bounds=(0.0,None), softbounds=(0.0,1.0), 
+        doc="Thickness (width) of the latency band for every presentation.")
+        
+    min_latency = param.Integer(default=0, bounds=(0,None), inclusive_bounds=(True,False),
+        doc="""Smallest latency on the generator sheet.""")
+
+    max_latency = param.Integer(default=100, bounds=(0,None), inclusive_bounds=(False,False),
+        doc="""Largest latency on the generator sheet.""")
+
+
+    def getLatency(self):    
+        latency_range = self.max_latency - self.min_latency
+        
+        sheet_range = self.bounds.lbrt()
+        x_range = sheet_range[2] - sheet_range[0]
+        
+        ratio = latency_range / x_range
+        
+        return ((self.x - sheet_range[0]) * ratio) + self.min_latency
+        
+        
+    def setLatency(self, new_latency):
+        latency_range = self.max_latency - self.min_latency
+        
+        sheet_range = self.bounds.lbrt()
+        x_range = sheet_range[2] - sheet_range[0]
+        
+        ratio = latency_range / x_range
+        
+        x = ((new_latency-self.min_latency) / ratio) + sheet_range[0]
+        setattr(self, 'x', x)
+
+
+    latency = property(getLatency, setLatency, "The latency at which to present.")           
+    
+    
+    def function(self, p):
+        return line(self.pattern_x, p.size, 0.001)
+
+
+
+class measure_latency_preference(MeasureResponseCommand):
+    """Measure a best onset latency preference and selectivity map for auditory neurons."""
+        
+    display = param.Boolean(True) 
+    static_parameters = param.List(default=["scale", "offset"])        
+
+    def _feature_list(self,p):
+        input_sheets = topo.sim.objects(GeneratorSheet).values()
+        
+        # BK-NOTE: if anyone wants to generalise this method for heterogeneous sheets, by all means do so,
+        # for my personal use the additional code required to generalise was overkill.
+        for sheet in range(1, len(input_sheets)-1):
+            assert input_sheets[sheet].bounds == input_sheets[sheet-1].bounds
+            assert input_sheets[sheet].xdensity == input_sheets[sheet-1].xdensity
+            assert input_sheets[sheet].ydensity == input_sheets[sheet-1].ydensity
+        
+        divisions = float(input_sheets[0].xdensity)
+
+        try:
+            min_latency = input_sheets[0].input_generator.min_latency
+            max_latency = input_sheets[0].input_generator.max_latency
+        except AttributeError:
+            min_latency = 0
+            max_latency = int(divisions)
+            self.warning("Input generator is missing min_latency or max_latency, setting to", str(min_latency), "&", str(max_latency),"respectively.")
+
+        generator = latency_mapper(size=1.0/divisions, min_latency=min_latency, max_latency=max_latency)
+        self.pattern_presenter = PatternPresenter2(pattern_generator=generator)
+        
+        return [Feature(name="latency", range=(min_latency,max_latency), step=1)]
+
+
+pg= create_plotgroup(name='Latency Preference and Selectivity', category="Auditory",
+    pre_plot_hooks=[measure_latency_preference.instance()], normalize='Individually',
+    doc='Measure a best onset latency preference and selectivity map for auditory neurons.')
+
+pg.add_plot('[Latency Preference]', [('Strength','LatencyPreference')])
+pg.add_plot('[Latency Selectivity]', [('Strength','LatencySelectivity')])
 
-pg.add_plot('[Frequency Preference]', [('Strength','YPreference')])
-pg.add_plot('[Frequency Selectivity]', [('Strength','YSelectivity')])
 
 
 import types
@@ -808,5 +1266,10 @@ __all__ = list(set([k for k,v in locals().items()
                     if isinstance(v,types.FunctionType) or 
                     (isinstance(v,type) and issubclass(v,ParameterizedFunction))
                     and not v.__name__.startswith('_')]))
-
-
+__all__ += [
+    "PatternPresenter2",
+    "frequency_mapper",
+    "gaussian_corner",
+    "latency_mapper",
+    "log_frequency_mapper",
+]
