@@ -4,9 +4,8 @@ the Topographica command prompt.  Typically called from the
 './topographica' script, but can be called directly if using
 Topographica files within a separate Python.
 
-$Id: commandline.py 11144 2010-07-09 17:55:14Z ceball $
+$Id$
 """
-__version__='$Revision: 11144 $'
 
 
 from optparse import OptionParser
@@ -14,12 +13,13 @@ from optparse import OptionParser
 import sys, __main__, math, os, re
 
 import topo
-from param.parameterized import Parameterized,OptionalSingleton
+import param
+from param.parameterized import Parameterized
+from topo.base.simulation import OptionalSingleton
 
 
 try:
     # By default, use a non-GUI backend for matplotlib.
-    import matplotlib
     from matplotlib import rcParams
     rcParams['backend']='Agg'
     matplotlib_imported=True
@@ -27,12 +27,24 @@ except ImportError:
     matplotlib_imported=False
 
 
+ipython_shell_interface = None
+ipython_prompt_interface = None
 try:
-    import IPython
-    ipython_imported=True
+    from IPython.frontend.terminal.embed import InteractiveShellEmbed as IPShell
+    from IPython.config.loader import Config
+    ipython_shell_interface = "InteractiveShellEmbed"
+    try:
+        from IPython.core.prompts import PromptManager  # pyflakes:ignore (try/except import)
+        ipython_prompt_interface = "PromptManager"
+    except ImportError:
+        pass
 except ImportError:
-    ipython_imported=False
-    print "Note: IPython is not available; using basic interactive Python prompt instead."
+    try:
+        # older version?
+        from IPython.Shell import IPShell  # pyflakes:ignore (try/except import)
+        ipython_shell_interface = "IPython.Shell"
+    except ImportError:
+        print "Note: IPython is not available; using basic interactive Python prompt instead."
 
 
 
@@ -221,14 +233,18 @@ class CommandPrompt(IPCommandPromptHandler):
       # Print the command number but don't use color:
       CommandPrompt.set_format('\N ')
       # Print the value of my_var at each prompt:
-      CommandPrompt.set_format('${my_var}>>> ')        
+      CommandPrompt.set_format('{my_var}>>> ')
     """
     _prompt = 'prompt1'
     
     # Predefined alternatives
     basic_format   = 'Topographica>>> '
-    simtime_format = 'topo_t${topo.sim.timestr()}>>> '
-    simtimecmd_format = 'topo_t${topo.sim.timestr()}_c\\#>>> '
+    if ipython_prompt_interface == "PromptManager":
+        simtime_format = 'topo_t{topo.sim.timestr_prop}>>> '
+        simtimecmd_format = 'topo_t{topo.sim.timestr_prop}_c\\#>>> '
+    else:
+        simtime_format = 'topo_t${topo.sim.timestr()}>>> '
+        simtimecmd_format = 'topo_t${topo.sim.timestr()}_c\\#>>> '
     
     _format = simtimecmd_format
 
@@ -333,7 +349,6 @@ launch Topographica with legacy support enabled.""")
 def gui(start=True):
     """Start the GUI as if -g were supplied in the command used to launch Topographica."""
     if matplotlib_imported: 
-        from matplotlib import rcParams
         rcParams['backend']='TkAgg'
     auto_import_commands()
     if start:
@@ -355,6 +370,9 @@ launch an interactive graphical user interface; \
 equivalent to -c 'from topo.misc.commandline import gui ; gui()'. \
 Implies -a.""")
 
+topo_parser.add_option("--pdb",action="store_true",dest="pdb",help="""\
+Automatically call the pdb debugger after every uncaught \
+exception. See IPython documentation for further details.""")
 
 # Keeps track of whether something has been performed, when deciding whether to assume -i
 something_executed=False
@@ -365,7 +383,11 @@ def c_action(option,opt_str,value,parser):
     exec value in __main__.__dict__
     global something_executed
     something_executed=True
-            
+    openmp_settings_names = ['openmp_threads', 'openmp_min_threads', 'openmp_max_threads']
+    openmp_present = [(k in __main__.__dict__) for k in openmp_settings_names]
+    if openmp_present and parser.values.gui: 
+        print "\nWARNING: For OpenMP settings to be used properly they need to be specified after the -g flag."
+    
 topo_parser.add_option("-c","--command",action = "callback",callback=c_action,type="string",
 		       default=[],dest="commands",metavar="\"<command>\"",
 		       help="string of arbitrary Python code to be executed in the main namespace.")
@@ -396,6 +418,7 @@ def auto_import_commands():
         if re.match('^[^_.].*\.py$',f):
             modulename = re.sub('\.py$','',f)
             exec "from topo.command."+modulename+" import *" in __main__.__dict__
+    exec "from topo.command import *" in __main__.__dict__
     
 def a_action(option,opt_str,value,parser):
     """Callback function for the -a option."""
@@ -409,47 +432,88 @@ equivalent to -c 'from topo.misc.commandline import auto_import_commands ; auto_
 
 def exec_startup_files():
     """
-    Execute startup files, looking at appropriate locations for many different platforms.
+    Execute startup files.
+
+    Linux/UNIX/OS X: ~/.topographicarc
+    Windows: %USERPROFILE%\topographica.ini
     """
-    home = os.path.expanduser("~")  # dotfiles on unix
-    appdata = os.path.expandvars("$APPDATA") # application data on windows
-    appsupport = os.path.join(home,"Library","Application Support") # application support on OS X  
+    # From Bilal: On OS X, ~/Library/Preferences/ is the standard path
+    # for user-defined params. The filename format (corresponding to
+    # .ini on windows) is org.topographica.plist, where a plist is an
+    # XML file. But, many shell-based programs follow the Unix
+    # convention, so we should be fine doing that.
 
-    rcpath = os.path.join(home,'.topographicarc')
-    inipath = os.path.join(appdata,'Topographica','topographica.ini')
-    configpath = os.path.join(appsupport,'Topographica','topographica.config')
+    # Linux/UNIX/OS X:
+    rcpath = os.path.join(os.path.expanduser("~"),'.topographicarc')
+    # Windows (ini is convention, and can be double clicked to edit):
+    inipath = os.path.join(os.path.expandvars("$USERPROFILE"),'topographica.ini')
 
-    for startup_file in (rcpath,configpath,inipath):
+    for startup_file in (rcpath,inipath):
         if os.path.exists(startup_file):
             print "Executing user startup file %s" % (startup_file)
             execfile(startup_file,__main__.__dict__)
-                
-
-    ### Notes about choices for topographica.rc equivalents on different platforms
-    #
-    ## Windows:
-    # Location --  Most programs use the registry or a folder in %appdata% (which is typically
-    #  ~\Application Data). The registry is not easily accessible for users, and %appdata% is
-    # a hidden folder, which means it doesn't appear in Explorer (or file-open dialogs).
-    # Most programs do not have any user-editable configuration files, so this does not matter
-    # to them. Maybe we should just use ~\topographica.ini?
-    # 
-    # Name -- Considered topographica.rc, topographica.dat, topographica.cfg, topographica.ini.
-    #  Of those, only .ini is registered as standard in Windows. According to Winows Explorer:
-    #  "Files with extension 'INI' are of type 'Configuration Settings'"
-    #  Importantly, this means they are already setup to be editable by notepad by default, so
-    #  they can be double clicked.
-    #
-    # http://mail.python.org/pipermail/python-list/2005-September/341702.html
-    #
-    ## Mac OS:
-    # Location -- Seems like programs use either ~/Library/AppName or (more commonly)
-    # ~/Library/Application Support/AppName (CEBALERT: is there a var. for that on OS X?).
-    # 
-    # Name -- there are many different extensions (e.g. dat, config, cfg, ini), none of which
-    # opens with any application by default. Some applications use xml.
+    
+    #####
+    # CEBALERT: locations we used to use on Windows and OS X. Should
+    # remove after 0.9.8.
+    # application data on windows
+    inipath = os.path.join(os.path.expandvars("$APPDATA"),'Topographica','topographica.ini')
+    # application support on OS X  
+    configpath = os.path.join(os.path.expanduser("~"),"Library","Application Support",'Topographica','topographica.config')
+    for startup_file in (configpath,inipath):
+        if os.path.exists(startup_file):
+            param.Parameterized().warning("Ignoring %s; location for startup file is %s (UNIX/Linux/Mac OS X) or %s (Windows)."%(startup_file,rcpath,inipath)) 
+    #####
 
 
+def get_omp_num_threads(openmp_threads, openmp_min_threads, openmp_max_threads):
+    """ Helper function to implement sensible OpenMP behaviour where
+        possible.  Returns a integer tuple (OpenMP threads, CPUs detected).
+
+        CPUs detected may be None if multiprocessing is not available (ie
+        Python 2.5) and consequently OpenMP threads may be None if the user
+        gave a relative thread specification (ie openmp_threads <= 0). When
+        OpenMP threads is None, the default OpenMP behaviour of using all
+        available cores is used.
+        """
+
+    if 'OMP_NUM_THREADS' in os.environ: return (None, None)
+
+    if 'NSLOTS' in os.environ: return ('NSLOTS', None)
+
+    try:
+        import multiprocessing
+        total_cores = multiprocessing.cpu_count()
+        if total_cores == 1: return (1, 1)
+        if total_cores == 2: return (2, 2)
+    except:
+        print "Cannot import multiprocessing to determine number of cores."
+        total_cores = None
+
+    if total_cores and (-openmp_threads >= (total_cores-1)):
+        print "OpenMP: Topographica needs a positive number of threads to execute!"
+        return (1, total_cores)
+
+    if (openmp_threads <= 0) and total_cores:      openmp_threads = total_cores + openmp_threads
+    if (openmp_threads <= 0) and not total_cores:  return (None, total_cores)
+
+    if (total_cores is not None) and (openmp_threads > total_cores):
+        print "OpenMP: More threads specified than cores detected."; return (total_cores, total_cores)
+
+    if openmp_threads < openmp_min_threads:
+        print "OpenMP: Using minimum number of allowed threads."
+        openmp_threads = openmp_min_threads
+
+    if (openmp_max_threads is None): return (openmp_threads, total_cores)
+
+    elif (openmp_max_threads < openmp_min_threads):
+        print"OpenMP: Maximum allowed threads lower than minimum allowed threads. Ignoring maximum limit."
+        return (openmp_threads, total_cores)
+    elif (openmp_threads > openmp_max_threads):
+        print "OpenMP: Using maximum number of allowed threads."
+        return (openmp_max_threads, total_cores)
+    else:
+        return (openmp_threads, total_cores)
 
 
 ### Execute what is specified by the options.
@@ -462,7 +526,7 @@ def process_argv(argv):
     import __main__
     for (k,v) in global_constants.items():
         exec '%s = %s' % (k,v) in __main__.__dict__
-    
+
     exec_startup_files()
 
     # Repeatedly process options, if any, followed by filenames, if any, until nothing is left
@@ -484,37 +548,90 @@ def process_argv(argv):
 
             execfile(filename,__main__.__dict__)
             something_executed=True
-            
+
         if not args:
             break
 
     global_params.check_for_unused_names()
 
+    # OpenMP settings and defaults
+    openmp_threads = __main__.__dict__.get('openmp_threads')
+    if (openmp_threads is None): openmp_threads=-1
+
+    openmp_min_threads = __main__.__dict__.get('openmp_min_threads')
+    if (openmp_min_threads is None): openmp_min_threads=2
+
+    openmp_max_threads = __main__.__dict__.get('openmp_max_threads')
+
+    if (openmp_threads != 1): # OpenMP is disabled if openmp_threads == 1
+
+        (num_threads, total_cores) = get_omp_num_threads(openmp_threads,
+                                                         openmp_min_threads,
+                                                         openmp_max_threads)
+
+        if num_threads is None:
+            print "OpenMP: Using OMP_NUM_THREADS environment variable if set. Otherwise, all cores in use."
+        elif num_threads == 'NSLOTS':
+            os.environ['OMP_NUM_THREADS'] =  os.environ['NSLOTS']
+            print "NSLOTS environment variable found; overriding any other thread settings and using N=%s threads" % os.environ['NSLOTS']
+
+        elif total_cores is None:
+            print "OpenMP: Using %d threads" % num_threads
+            os.environ['OMP_NUM_THREADS'] =  str(num_threads)
+        else:
+            print "OpenMP: Using %d threads on a machine with %d detected CPUs" % (num_threads, total_cores)
+            os.environ['OMP_NUM_THREADS'] =  str(num_threads)
+
     # If no scripts and no commands were given, pretend -i was given.
     if not something_executed: interactive()
-     
+
     if option.gui: topo.guimain.title(topo.sim.name)
 
     ## INTERACTIVE SESSION BEGINS HERE (i.e. can't have anything but
     ## some kind of cleanup code afterwards)
     if os.environ.get('PYTHONINSPECT'):
+        print "Output path: %s" % param.normalize_path.prefix
         print BANNER    
-        # CB: should probably allow a way for users to pass things to
-        # IPython? Or at least set up some kind of topographica ipython
-        # config file
+        # CBALERT: should probably allow a way for users to pass
+        # things to IPython? Or at least set up some kind of
+        # topographica ipython config file. Right now, a topo_parser
+        # option has to be added for every ipython option we want to
+        # support (e.g. see --pdb)
+
+        if ipython_shell_interface == "IPython.Shell":
+            # IPython 0.10 and earlier
 
-        if ipython_imported:
             # Stop IPython namespace hack?
             # http://www.nabble.com/__main__-vs-__main__-td14606612.html
             __main__.__name__="__mynamespace__"
 
-            from IPython.Shell import IPShell
+            ipython_args = ['-noconfirm_exit','-nobanner',
+                            '-pi1',CommandPrompt.get_format(),
+                            '-pi2',CommandPrompt2.get_format(),
+                            '-po',OutputPrompt.get_format()]
+            if option.pdb:
+                ipython_args.append('-pdb')
 
-            IPShell(['-noconfirm_exit','-nobanner',
-                     '-pi1',CommandPrompt.get_format(),
-                     '-pi2',CommandPrompt2.get_format(),
-                     '-po',OutputPrompt.get_format()],
-                    user_ns=__main__.__dict__).mainloop(sys_exit=1)            
+            ipshell = IPShell(ipython_args,user_ns=__main__.__dict__)
+            ipshell.mainloop(sys_exit=1)
 
-        
+        elif ipython_shell_interface == "InteractiveShellEmbed":
+            # IPython 0.11 and later
 
+            config = Config()
+
+            if ipython_prompt_interface == "PromptManager":
+                config.PromptManager.in_template = CommandPrompt.get_format()
+                config.PromptManager.in2_template = CommandPrompt2.get_format()
+                config.PromptManager.out_template = OutputPrompt.get_format()
+            else:
+                config.InteractiveShell.prompt_in1 = CommandPrompt.get_format()
+                config.InteractiveShell.prompt_in2 = CommandPrompt2.get_format()
+                config.InteractiveShell.prompt_out = OutputPrompt.get_format()
+            config.InteractiveShell.confirm_exit = False
+            ipshell = IPShell(config=config,user_ns=__main__.__dict__,
+                              banner1="",exit_msg="")
+            if option.pdb:
+                ipshell.call_pdb = True
+            ipshell()
+            sys.exit()
