@@ -5,9 +5,8 @@ plots.
 Includes PlotGroups for standard plots of anything in a Sheet
 database, plus weight plots for one unit, and projections.
 
-$Id: plotgroup.py 11137 2010-07-07 03:34:18Z ceball $
+$Id$
 """
-__version__='$Revision: 11137 $'
 
 import copy
 import Image
@@ -18,7 +17,7 @@ from param import resolve_path
 
 import topo
 
-from topo.base.cf import CFSheet,CFProjection,Projection
+from topo.base.cf import CFSheet,CFProjection
 from topo.base.projection import ProjectionSheet
 from topo.sheet import GeneratorSheet,Sheet
 from topo.misc.keyedlist import KeyedList
@@ -544,7 +543,7 @@ class TemplatePlotGroup(SheetPlotGroup):
         self.plot_templates = KeyedList(plot_templates or [])
         # Add plots for the static images, if any
         for image_name,file_path in static_images or []:
-            add_static_image(image_name,file_path)
+            self.add_static_image(image_name,file_path)
 
 
     #########################
@@ -987,6 +986,28 @@ class RFProjectionPlotGroup(GridPlotGroup):
         super(RFProjectionPlotGroup,self)._exec_pre_plot_hooks(input_sheet=self.input_sheet,**kw)
 
 
+class TwoOrientationsPlotGroup( TemplatePlotGroup ):
+    """Display with small segments the two most preferred orientations for each
+    units in the sheet. Only orientation with significative selectivity are
+    shown. Darker segments correspond to orientations with more selective
+    response."""
+
+    unit_size       = param.Number(default=25,bounds=(9,None),doc="box size of a single unit")
+
+    # Overrides:
+    # _make_template_plot	- use density argument slot to parse unit_size
+
+    def _make_template_plot(self,plot_template_name,plot_template,**kw):
+        return make_template_plot(plot_template,
+                                  kw['sheet'].sheet_views,
+                                  self.unit_size,
+                                  kw['sheet'].bounds,
+                                  self.normalize,
+                                  name=plot_template_name,
+                                  range_=kw['range_'])
+
+
+
 
 # CEBALERT: haven't modified; might need updating. Doesn't seem to be
 # a way to call it from the GUI - is it known to be broken?
