@@ -15,12 +15,18 @@ other CFPResponseFns beyond CFPRF_Plugin.
 Any new response functions added to this directory will automatically
 become available for any model.
 
-$Id: __init__.py 8937 2008-08-21 13:28:43Z ceball $
+$Id$
 """
-__version__='$Revision: 8937 $'
 
-# Automatically discover all .py files in this directory, and import classes from basic.py. 
+from topo.base.functionfamily import ResponseFn
+
+# Imported here so that all ResponseFns will be in the same package
+from topo.base.cf import DotProduct  # pyflakes:ignore (API import)
+
+_public = list(set([_k for _k,_v in locals().items()
+                    if isinstance(_v,type) and issubclass(_v,ResponseFn)]))
+
+# Automatically discover all .py files in this directory.
 import os,fnmatch
-from basic import *
-__all__ = basic.__all__ + [f.split('.py')[0] for f in os.listdir(__path__[0]) if fnmatch.fnmatch(f,'[!._]*.py')]
+__all__ = _public + [f.split('.py')[0] for f in os.listdir(__path__[0]) if fnmatch.fnmatch(f,'[!._]*.py')]
 del f,os,fnmatch
