@@ -1,8 +1,7 @@
 """
 
-$Id: testimage.py 11137 2010-07-07 03:34:18Z ceball $
+$Id$
 """
-__version__='$Revision: 11137 $'
 
 # CEBALERT: incomplete - being written.
 
@@ -27,7 +26,7 @@ from param import resolve_path
 
 from topo.base.boundingregion import BoundingBox
 from topo.pattern.image import FileImage
-from topo.transferfn.basic import IdentityTF
+from topo.transferfn import IdentityTF
 
 
 
