@@ -2,34 +2,26 @@
 Classes providing GUI windows for PlotGroups, allowing sets of related plots
 to be displayed.
 
-$Id: plotgrouppanel.py 11111 2010-07-05 10:53:13Z ceball $
+$Id$
 """
-__version__='$Revision: 11111 $'
 
 
 import copy
 
-from inspect import getdoc
-from math import floor
-
 import ImageTk
 
 import Tkinter
-from Tkinter import  Frame, TOP, YES, BOTH, BOTTOM, X, Button, LEFT, \
-     RIGHT, DISABLED, Checkbutton, NORMAL, Canvas, Label, NSEW, IntVar, \
-     BooleanVar, StringVar, FLAT, SUNKEN, RAISED, GROOVE, RIDGE, \
-     Scrollbar, Y, VERTICAL, HORIZONTAL, END, NO, NONE,Scrollbar,Canvas, \
-     TclError
+from Tkinter import  Frame, TOP, YES, BOTH, X, LEFT, \
+     RIGHT, DISABLED, NORMAL, Canvas, Label, NSEW, \
+     NO, NONE,TclError
 
 import param
-# CEB: maybe bad, since people reading code will probably assume tk is Tkinter
-from param import tk
+# CEBALERT: here and elsewhere, change tk to paramtk
+import paramtk as tk
 
 from topo.base.sheet import Sheet
 from topo.base.cf import CFSheet
 
-from topo.plotting.plotgroup import PlotGroup,SheetPlotGroup
-
 from topo.command.pylabplot import matrixplot
 
 from topo.misc.generatorsheet import GeneratorSheet
@@ -331,7 +323,6 @@ Many hooks accept 'display=True' so that the progress can be viewed in an open A
                 label="Combined plot: %s %s"%(plot.plot_src_name,plot.name),
                 state=NORMAL)            
             (r,c),(x,y) = event_info['coords']
-            sheet = topo.sim[plot.plot_src_name]    
             self._canvas_menu.entryconfig("unit_menu",
                 label="Single unit:(% 3d,% 3d) Coord:(% 2.2f,% 2.2f)"%(r,c,x,y),
                 state=NORMAL)
@@ -647,7 +638,7 @@ Many hooks accept 'display=True' so that the progress can be viewed in an open A
 
         # CEBALERT: I guess some widgets don't have state?
         try:
-            current_state = widget.configure('state')[3]
+            current_state = widget.configure('state')[3]  # pyflakes:ignore (try/except test)
         except TclError:
             return
 
